// SPDX-License-Identifier: (GPL-2.0+ OR MIT)
/*
 * Copyright (c) 2019 Amlogic, Inc. All rights reserved.
 */

#include <common.h>
#include <command.h>
#include <linux/arm-smccc.h>

#define DMC_MON_RW			0x8200004A

#define DMC_SEC_STATUS                             ((0x00b8  << 2) + 0xff639000)
#define DMC_VIO_ADDR0                              ((0x00b9  << 2) + 0xff639000)
#define DMC_VIO_ADDR1                              ((0x00ba  << 2) + 0xff639000)
#define DMC_VIO_ADDR2                              ((0x00bb  << 2) + 0xff639000)
#define DMC_VIO_ADDR3                              ((0x00bc  << 2) + 0xff639000)

unsigned long dmc_rw(u64 addr, u64 value, u64 rw)
{
	struct arm_smccc_res res;

	arm_smccc_smc(DMC_MON_RW, addr, value, rw, 0, 0, 0, 0, &res);

	return res.a0;
}

int do_dmc_rw(struct cmd_tbl *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc < 4 || argc > 5)
		return cmd_usage(cmdtp);

	u64 addr = simple_strtoul(argv[1], NULL, 16);
	u64 value = simple_strtoul(argv[2], NULL, 16);
	u64 rw = simple_strtoul(argv[3], NULL, 16);
	u64 dmc_back;

	dmc_back = dmc_rw(addr, value, rw);

	printf("return value is %llx\n", dmc_back);

	return dmc_back;
}

static char dmc_help_text[] =
	"\n"
	"dmc_rw fe037470 ffffffff 0 - show dmc read information\n"
	"dmc_rw fe037470 ffffffff 1 - show dmc write information\n";

U_BOOT_CMD(dmc_rw, 4, 0, do_dmc_rw,
	"dmc read write function",
	dmc_help_text
);

static int dmc_sec_check(unsigned int dmc_number)
{
#define DMC_READ	0
#define DMC_WRITE	1

#if 0
	cpu_id_t cpu_id = get_cpu_id();
#endif

	unsigned int dmc_vio_status = dmc_rw(DMC_SEC_STATUS, 0, DMC_READ);
	unsigned int dmc_vio_reg0 = dmc_rw(DMC_VIO_ADDR0, 0, DMC_READ);
	unsigned int dmc_vio_reg1 = dmc_rw(DMC_VIO_ADDR1, 0, DMC_READ);
	unsigned int dmc_vio_reg2 = dmc_rw(DMC_VIO_ADDR2, 0, DMC_READ);
	unsigned int dmc_vio_reg3 = dmc_rw(DMC_VIO_ADDR3, 0, DMC_READ);
	unsigned int dmc_vio_reg4 = 0;
	unsigned int dmc_vio_reg5 = 0;
	unsigned int dmc_vio_reg6 = 0;
	unsigned int dmc_vio_reg7 = 0;

#if 0
	if (cpu_id.family_id == MESON_CPU_MAJOR_ID_GXBB ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_GXTVBB ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_GXL ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_GXM ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_TXL ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_TXLX ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_AXG ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_GXLX ||
	cpu_id.family_id == MESON_CPU_MAJOR_ID_TXHD) {
		dmc_vio_reg4 = dmc_rw(DMC_VIO_ADDR3 + 4, 0, DMC_READ);
		dmc_vio_reg5 = dmc_rw(DMC_VIO_ADDR3 + 8, 0, DMC_READ);
		dmc_vio_reg6 = dmc_rw(DMC_VIO_ADDR3 + 12, 0, DMC_READ);
		dmc_vio_reg7 = dmc_rw(DMC_VIO_ADDR3 + 16, 0, DMC_READ);
	}
#endif

	if (dmc_vio_status || dmc_vio_reg0 || dmc_vio_reg1 || dmc_vio_reg2 || dmc_vio_reg3 ||
	    dmc_vio_reg4 || dmc_vio_reg5 || dmc_vio_reg6 || dmc_vio_reg7) {
		printf("WARNING: DMC%d SEC VIOLATION CHECK FAILED!!!\n", dmc_number);
		printf("DMC%d_SEC_STATUS:%d\n", dmc_number, dmc_vio_status);
		printf("DMC%d_VIO_ADDR0:%x\n", dmc_number, dmc_vio_reg0);
		printf("DMC%d_VIO_ADDR1:%x\n", dmc_number, dmc_vio_reg1);
		printf("DMC%d_VIO_ADDR2:%x\n", dmc_number, dmc_vio_reg2);
		printf("DMC%d_VIO_ADDR3:%x\n", dmc_number, dmc_vio_reg3);

#if 0
		if (cpu_id.family_id == MESON_CPU_MAJOR_ID_GXBB ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_GXTVBB ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_GXL ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_GXM ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_TXL ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_TXLX ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_AXG ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_GXLX ||
		cpu_id.family_id == MESON_CPU_MAJOR_ID_TXHD) {
			printf("DMC%d_VIO_ADDR4:%x\n", dmc_number, dmc_vio_reg4);
			printf("DMC%d_VIO_ADDR5:%x\n", dmc_number, dmc_vio_reg5);
			printf("DMC%d_VIO_ADDR6:%x\n", dmc_number, dmc_vio_reg6);
			printf("DMC%d_VIO_ADDR7:%x\n", dmc_number, dmc_vio_reg7);
		}
#endif
	} else {
		printf("DMC%d SEC VIOLATION CHECK PASS.\n", dmc_number);
	}
	dmc_rw(DMC_SEC_STATUS, 0x3, DMC_WRITE);

	return 0;
}

int do_dmc_sec_vio_check(struct cmd_tbl *cmdtp, int flag, int argc, char * const argv[])
{
	dmc_sec_check(0);
	return 0;
}

static char dmc_vio_check_help_text[] =
	"\n"
	"dmc_vio_check - show dmc sec vio reg check information\n";

U_BOOT_CMD(dmc_vio_check, 1, 0, do_dmc_sec_vio_check,
	"dmc sec violation reg check",
	dmc_vio_check_help_text
);
